@@ -81,12 +81,12 @@ func TestFactoryLoadContainer(t *testing.T) {
 	}
 }
 
-func marshal(path string, v interface{}) error {
+func marshal(path string, v any) error {
 	f, err := os.Create(path)
 	if err != nil {
 		return err
 	}
-	defer f.Close() //nolint: errcheck
+	defer f.Close()
 	return utils.WriteJSON(f, v)
 }
 
